package ru.landgrafhomyak.serdha.api.v0

import kotlinx.datetime.Instant
import kotlinx.datetime.LocalDate
import kotlinx.datetime.LocalDateTime
import kotlinx.datetime.LocalTime
import ru.landgrafhomyak.serdha.api.v0.ddl.types.DatabaseType

public interface Expression<RuntimeType, DT : DatabaseType<RuntimeType>, OwnerBuilderUserExtension : Any> {
	public interface Builder<OwnerBuilderUserExtension : Any> {
		public fun <DT : DatabaseType<*>> equals(
			left: Expression<*, DT, OwnerBuilderUserExtension>,
			right: Expression<*, DT, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun <DT : DatabaseType<*>> notEquals(
			left: Expression<*, DT, OwnerBuilderUserExtension>,
			right: Expression<*, DT, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension> =
			this.logicalNot(this.equals(left, right))

		public fun isNull(
			e: Expression<*, DatabaseType.Nullable<*, *>, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun isNotNull(
			e: Expression<*, DatabaseType.Nullable<*, *>, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension> =
			this.logicalNot(this.isNull(e))

		public fun logicalAnd(
			left: Expression<*, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
			right: Expression<*, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun logicalOr(
			left: Expression<*, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
			right: Expression<*, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun logicalXor(
			left: Expression<*, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
			right: Expression<*, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension> =
			this.notEquals(left, right)

		public fun logicalNot(
			e: Expression<*, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
			right: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
			right: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
			right: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
			right: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
			right: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
			right: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>,
			right: Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>,
			right: Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
			right: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
			right: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
			right: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
			right: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
			right: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
			right: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>,
			right: Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>,
			right: Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
			right: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
			right: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
			right: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
			right: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
			right: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
			right: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>,
			right: Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>,
			right: Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
			right: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
			right: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
			right: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
			right: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
			right: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
			right: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>,
			right: Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>,
			right: Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun extendIntegerToS16(
			e: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>

		public fun extendIntegerToS16(
			e: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>

		public fun extendIntegerToU16(
			e: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>

		public fun extendIntegerToS32(
			e: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>

		public fun extendIntegerToS32(
			e: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>

		public fun extendIntegerToS32(
			e: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>

		public fun extendIntegerToS32(
			e: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>

		public fun extendIntegerToU32(
			e: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>

		public fun extendIntegerToU32(
			e: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToU64(
			e: Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>

		public fun extendIntegerToU64(
			e: Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>

		public fun extendIntegerToU64(
			e: Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>

		public fun extendFractionalToDouble(
			e: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>
		): Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>

		public fun cmpFractionalLess(
			left: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>,
			right: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalLess(
			left: Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>,
			right: Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalLessOrEquals(
			left: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>,
			right: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalLessOrEquals(
			left: Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>,
			right: Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalGreater(
			left: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>,
			right: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalGreater(
			left: Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>,
			right: Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalGreaterOrEquals(
			left: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>,
			right: Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalGreaterOrEquals(
			left: Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>,
			right: Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateEarlier(
			left: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
			right: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateEarlierOrSimultaneous(
			left: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
			right: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateLater(
			left: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
			right: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateLaterOrSimultaneous(
			left: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
			right: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalTimeEarlier(
			left: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
			right: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalTimeEarlierOrSimultaneous(
			left: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
			right: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalTimeLater(
			left: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
			right: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalTimeLaterOrSimultaneous(
			left: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
			right: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateTimeEarlier(
			left: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
			right: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateTimeEarlierOrSimultaneous(
			left: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
			right: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateTimeLater(
			left: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
			right: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateTimeLaterOrSimultaneous(
			left: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
			right: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpInstantEarlier(
			left: Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>,
			right: Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpInstantEarlierOrSimultaneous(
			left: Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>,
			right: Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpInstantLater(
			left: Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>,
			right: Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpInstantLaterOrSimultaneous(
			left: Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>,
			right: Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun dateComponentOf(
			dt: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>

		public fun timeComponentOf(
			dt: Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>


		public fun joinDateTime(
			date: Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>,
			time: Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>


		public fun constantOf(v: Boolean): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>
		public fun constantOf(v: Byte): Expression<Byte, DatabaseType.INT_S8, OwnerBuilderUserExtension>
		public fun constantOf(v: UByte): Expression<UByte, DatabaseType.INT_U8, OwnerBuilderUserExtension>
		public fun constantOf(v: Short): Expression<Short, DatabaseType.INT_S16, OwnerBuilderUserExtension>
		public fun constantOf(v: UShort): Expression<UShort, DatabaseType.INT_U16, OwnerBuilderUserExtension>
		public fun constantOf(v: Int): Expression<Int, DatabaseType.INT_S32, OwnerBuilderUserExtension>
		public fun constantOf(v: UInt): Expression<UInt, DatabaseType.INT_U32, OwnerBuilderUserExtension>
		public fun constantOf(v: Long): Expression<Long, DatabaseType.INT_S64, OwnerBuilderUserExtension>
		public fun constantOf(v: ULong): Expression<ULong, DatabaseType.INT_U64, OwnerBuilderUserExtension>
		public fun constantOf(v: Float): Expression<Float, DatabaseType.FLOAT_32, OwnerBuilderUserExtension>
		public fun constantOf(v: Double): Expression<Double, DatabaseType.FLOAT_64, OwnerBuilderUserExtension>
		public fun constantOf(v: LocalDate): Expression<LocalDate, DatabaseType.LOCAL_DATE, OwnerBuilderUserExtension>
		public fun constantOf(v: LocalTime): Expression<LocalTime, DatabaseType.LOCAL_TIME, OwnerBuilderUserExtension>
		public fun constantOf(v: LocalDateTime): Expression<LocalDateTime, DatabaseType.LOCAL_DATETIME, OwnerBuilderUserExtension>
		public fun constantOf(v: Instant): Expression<Instant, DatabaseType.INSTANT, OwnerBuilderUserExtension>

		@OptIn(ExperimentalUnsignedTypes::class)
		public fun constantOf(v: ByteArray): Expression<UByteArray, DatabaseType.BINARY_DATA, OwnerBuilderUserExtension>

		public fun <ArgRT : Any, ArgDT : DatabaseType<ArgRT>, RetRuntimeType, RetDT : DatabaseType<RetRuntimeType>> ifNull(
			nullable: Expression<ArgRT?, DatabaseType.Nullable<ArgRT, ArgDT>, OwnerBuilderUserExtension>,
			onValue: (Expression<ArgRT, ArgDT, OwnerBuilderUserExtension>) -> Expression<RetRuntimeType, RetDT, OwnerBuilderUserExtension>,
			onNull: () -> Expression<RetRuntimeType, RetDT, OwnerBuilderUserExtension>
		): Expression<RetRuntimeType, RetDT, OwnerBuilderUserExtension>

		public fun <ArgRT : Any, ArgDT : DatabaseType<ArgRT>> notNullAnd(
			nullable: Expression<ArgRT?, DatabaseType.Nullable<ArgRT, ArgDT>, OwnerBuilderUserExtension>,
			onValue: (Expression<ArgRT, ArgDT, OwnerBuilderUserExtension>) -> Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>

		public fun <ArgRT : Any, ArgDT : DatabaseType<ArgRT>> nullOr(
			nullable: Expression<ArgRT?, DatabaseType.Nullable<ArgRT, ArgDT>, OwnerBuilderUserExtension>,
			onValue: (Expression<ArgRT, ArgDT, OwnerBuilderUserExtension>) -> Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, DatabaseType.BOOLEAN, OwnerBuilderUserExtension>
	}
}